@@ -29,10 +29,6 @@
 ## \code{\link[ecospat]{ecospat.mpa}}) # generate R CMD Check error due 
 ## to crossref missing ecospat package
 ##' 
-##' @param save.output (\emph{optional, default} \code{TRUE}) \cr 
-##' A \code{logical} value defining whether the output is to be saved within the 
-##' \code{.BIOMOD_DATA} folder or not
-##' 
 ##' 
 ##' @return 
 ##' 
@@ -125,11 +121,11 @@
 ##'                                       modeling.id = 'AllModels',
 ##'                                       models = c('RF', 'GLM'),
 ##'                                       bm.options = myBiomodOptions,
-##'                                       nb.rep = 2,
-##'                                       data.split.perc = 80,
+##'                                       CV.strategy = 'random',
+##'                                       CV.nb.rep = 2,
+##'                                       CV.perc = 0.8,
 ##'                                       metric.eval = c('TSS','ROC'),
 ##'                                       var.import = 3,
-##'                                       do.full.models = FALSE,
 ##'                                       seed.val = 42)
 ##' }
 ##' 
@@ -153,21 +149,20 @@
 ##' 
 ##' 
 ##' # --------------------------------------------------------------- #
-##' # Evaluate models with Boyce index and MPA
-##' myBiomodPO <- BIOMOD_PresenceOnly(bm.mod = myBiomodModelOut,
-##'                                   bm.em = myBiomodEM)
-##' myBiomodPO
-##' 
-##' # Evaluate models with Boyce index and MPA (using background data)
-##' myBiomodPO <- BIOMOD_PresenceOnly(bm.mod = myBiomodModelOut,
-##'                                   bm.em = myBiomodEM, 
-##'                                   bg.env = myExpl)
-##' myBiomodPO
+##' # # Evaluate models with Boyce index and MPA
+##' # myBiomodPO <- BIOMOD_PresenceOnly(bm.mod = myBiomodModelOut,
+##' #                                   bm.em = myBiomodEM)
+##' # myBiomodPO
+##' # 
+##' # # Evaluate models with Boyce index and MPA (using background data)
+##' # myBiomodPO <- BIOMOD_PresenceOnly(bm.mod = myBiomodModelOut,
+##' #                                   bm.em = myBiomodEM, 
+##' #                                   bg.env = myExpl)
+##' # myBiomodPO
 ##' 
 ##' 
 ## @importFrom ecospat ecospat.boyce ecospat.mpa
 ##' @importFrom PresenceAbsence presence.absence.accuracy
-##' @importFrom data.table rbindlist
 ##' @importFrom terra rast extract
 ##' 
 ##' @export
@@ -178,34 +173,36 @@
 BIOMOD_PresenceOnly <- function(bm.mod = NULL, 
                                 bm.em = NULL, 
                                 bg.env = NULL, 
-                                perc = 0.9, 
-                                save.output = TRUE)
+                                perc = 0.9)
 {
   .bm_cat("Do Presence-Only Evaluation")
   # if (!isNamespaceLoaded("ecospat")) { requireNamespace("ecospat", quietly = TRUE) }
   
   ## 0. Check arguments --------------------------------------------------------
-  args <- .BIOMOD_PresenceOnly.check.args(bm.mod, bm.em, bg.env, perc, save.output)
+  args <- .BIOMOD_PresenceOnly.check.args(bm.mod, bm.em, bg.env, perc)
   for (argi in names(args)) { assign(x = argi, value = args[[argi]]) }
   rm(args)
   
   
   ## 1. Get calib.lines ------------------------------------------------------
-  
   if (!is.null(bm.mod)) {
-    calib.lines <- get_calib_lines(bm.mod)[, , 1]
+    calib.lines <- get_calib_lines(bm.mod)#[, 1]
+    bm.data <- get_formal_data(bm.mod)
+    resp.var <- get_formal_data(bm.mod, subinfo = "resp.var")
   } else {
-    calib.lines <- get_calib_lines(get_formal_data(bm.em))[, , 1]
+    calib.lines <- get_calib_lines(get_formal_data(bm.em))#[, 1]
+    bm.data <- get_formal_data(get_formal_data(bm.em))
+    resp.var <- get_formal_data(get_formal_data(bm.em), subinfo = "resp.var")
   }
-  calib.notNA <- which(!is.na(calib.lines[, 1])) ## remove NA (pseudo-absences) from run1
-  calib.lines <- calib.lines[calib.notNA, ] ## keep only lines associated to sites (no pseudo-absences)
   
+
+  calib.notNA <- which(!is.na(resp.var))
+  calib.lines <- calib.lines[calib.notNA, ]
   
   ## 2. Individual models ----------------------------------
   if (!is.null(bm.mod)) {
     ## Get calibration lines and observations
-    myResp <- get_formal_data(bm.mod)@data.species
-    myResp <- myResp[calib.notNA] ## keep only lines associated to sites (no pseudo-absences)
+    myResp <- resp.var[calib.notNA] ## keep only lines associated to sites (no pseudo-absences)
     
     ## Get evaluation scores
     myEvalMod <- get_evaluations(bm.mod)
@@ -236,7 +233,21 @@ BIOMOD_PresenceOnly <- function(bm.mod = NULL,
     ## Get evaluation scores
     myEvalEM <- get_evaluations(bm.em)
     if (!is.null(bm.mod)) {
-      myEvalMod <- rbindlist(list(myEvalMod, myEvalEM), fill = TRUE)
+      full.colnames <- c("full.name", "PA", "run", "algo",
+                         "metric.eval", "cutoff", 
+                         "sensitivity", "specificity", "calibration", 
+                         "validation", "evaluation",
+                         "merged.by.PA", "merged.by.run", "merged.by.algo", "filtered.by")
+      
+      myEvalMod$merged.by.PA <- NA
+      myEvalMod$merged.by.run <- NA
+      myEvalMod$merged.by.algo <- NA
+      myEvalMod$filtered.by <- NA
+      myEvalEM$PA <- NA
+      myEvalEM$run <- NA
+      
+      myEvalMod <- rbind(myEvalMod[,full.colnames],
+                         myEvalEM[,full.colnames])
     } else {
       myEvalMod <- myEvalEM
     }
@@ -265,9 +276,9 @@ BIOMOD_PresenceOnly <- function(bm.mod = NULL,
           proj.name = paste(bm.em@modeling.id, "cv_EF_bg", sep = "_"))
       } else {
         myPredEM <- BIOMOD_EnsembleForecasting(
-            bm.em = bm.em,
-            new.env = bg.env,
-            proj.name = paste(bm.em@modeling.id, "cv_EF_bg", sep = "_"))
+          bm.em = bm.em,
+          new.env = bg.env,
+          proj.name = paste(bm.em@modeling.id, "cv_EF_bg", sep = "_"))
       }
       myPredEM <- .get_list_predictions(bm.out = myPredEM, evaluation = FALSE)
     }
@@ -295,20 +306,23 @@ BIOMOD_PresenceOnly <- function(bm.mod = NULL,
     ## Get model informations
     full.name <- boyce.eval[i, 1]
     tmp <- strsplit(as.character(full.name), split = "_")[[1]]
-    run <- tmp[c(grep("RUN", tmp), grep("allRun", tmp), grep("mergedRun", tmp))]
-    
+    length.tmp <- length(tmp)
+    # run <- tmp[c(grep("RUN", tmp), grep("allRun", tmp), grep("mergedRun", tmp))]
+    run <- 
+      paste0(
+        "_",
+        paste0(tmp[c(length.tmp - 2, length.tmp - 1)], 
+               collapse = "_")
+      )
+    # message(run)
     ## Get evaluation lines
     if (length(run) == 0) {
       ind.eval = NULL
     } else {
-      if (run == "mergedRun") {
+      if (grepl(run, pattern = "allRun") | grepl(run, pattern = "mergedRun")) {
         ind.eval = 1:nrow(calib.lines) 
       } else {
-        if (inherits(calib.lines, "matrix")) {
-          ind.eval = which(calib.lines[, paste0("_", run)] == FALSE)
-        } else {
-          ind.eval = which(calib.lines == FALSE)
-        }
+          ind.eval = which(!calib.lines[, run])
       }
     }
     
@@ -388,16 +402,14 @@ BIOMOD_PresenceOnly <- function(bm.mod = NULL,
   ## SAVE OUTPUTS ---------------------------------------------------------------------------------
   output <- rbind(myEvalMod, boyce.eval, mpa.eval)
   
-  if (save.output) {
-    if (!is.null(bm.mod)) {
-      sp <- bm.mod@sp.name
-      mod.id <- bm.mod@modeling.id
-    } else if (!is.null(bm.em)) {
-      sp <- bm.em@sp.name
-      mod.id <- bm.em@modeling.id
-    }
-    save(output, file = paste0(sp, "/.BIOMOD_DATA/", mod.id, "/presenceonly.evaluation_", sp))
+  if (!is.null(bm.mod)) {
+    sp <- bm.mod@sp.name
+    mod.id <- bm.mod@modeling.id
+  } else if (!is.null(bm.em)) {
+    sp <- bm.em@sp.name
+    mod.id <- bm.em@modeling.id
   }
+  save(output, file = paste0(sp, "/.BIOMOD_DATA/", mod.id, "/presenceonly.evaluation_", sp))
   
   .bm_cat("Done")
   return(output)
@@ -417,7 +429,7 @@ BIOMOD_PresenceOnly <- function(bm.mod = NULL,
 
 # Check Arguments -------------------------------------------------------------
 
-.BIOMOD_PresenceOnly.check.args <- function(bm.mod, bm.em, bg.env, perc, save.output)
+.BIOMOD_PresenceOnly.check.args <- function(bm.mod, bm.em, bg.env, perc)
 {
   
   if(is.null(bm.mod) && is.null(bm.em)){
@@ -457,7 +469,7 @@ BIOMOD_PresenceOnly <- function(bm.mod = NULL,
     
     if (inherits(bg.env, 'Raster')) {
       if(any(raster::is.factor(bg.env))){
-        bg.env <- categorical_stack_to_terra(bg.env)
+        bg.env <- .categorical_stack_to_terra(bg.env)
       } else {
         bg.env <- rast(bg.env)
       }
@@ -490,15 +502,14 @@ BIOMOD_PresenceOnly <- function(bm.mod = NULL,
   ## 5. Check MPA & Evaluation data ---------------------------------------------
   if ((!is.null(bm.mod) && bm.mod@has.evaluation.data) ||
       (is.null(bm.mod) && get_formal_data(bm.em)@has.evaluation.data)) {
-        cat("\n      ! Evaluation data will be ignored for MPA-related calculations")
-      }
-    
+    cat("\n      ! Evaluation data will be ignored for MPA-related calculations")
+  }
+  
   
   return(list(bm.mod = bm.mod,
               bm.em = bm.em,
               bg.env = bg.env,
-              perc = perc,
-              save.output = save.output))
+              perc = perc))
 }
 
 
