@@ -161,11 +161,11 @@
 ##'                                       modeling.id = 'AllModels',
 ##'                                       models = c('RF', 'GLM'),
 ##'                                       bm.options = myBiomodOptions,
-##'                                       nb.rep = 2,
-##'                                       data.split.perc = 80,
+##'                                       CV.strategy = 'random',
+##'                                       CV.nb.rep = 2,
+##'                                       CV.perc = 0.8,
 ##'                                       metric.eval = c('TSS','ROC'),
 ##'                                       var.import = 3,
-##'                                       do.full.models = FALSE,
 ##'                                       seed.val = 42)
 ##' }
 ##' 
@@ -187,7 +187,8 @@
 ##' 
 ##' @importFrom foreach foreach %dopar% 
 ## @importFrom doParallel registerDoParallel
-##' @importFrom terra rast subset nlyr writeRaster terraOptions wrap mem_info app
+##' @importFrom terra rast subset nlyr writeRaster terraOptions wrap unwrap
+##'  mem_info app is.factor
 ##' @importFrom utils capture.output
 ##' @importFrom abind asub
 ##' 
@@ -291,40 +292,39 @@ BIOMOD_Projection <- function(bm.mod,
   }
   
   proj <- foreach(mod.name = models.chosen) %dopar% {
-      cat("\n\t> Projecting", mod.name, "...")
-      if (do.stack) {
-        filename <- NULL
-      } else {
-        filename <- file.path(namePath, "individual_projections",
-                              paste0(nameProj, "_", mod.name, 
-                                     ifelse(output.format == ".RData"
-                                            , ".tif", output.format)))
-      }
-      
-      mod <- get(BIOMOD_LoadModels(bm.out = bm.mod, full.name = mod.name))
-      temp_workdir = NULL
-      if (length(grep("MAXENT$", mod.name)) == 1) {
-        temp_workdir = mod@model_output_dir
-      }
-      pred.tmp <- predict(mod, new.env, on_0_1000 = on_0_1000, 
-                          filename = filename, omit.na = omit.na, 
-                          temp_workdir = temp_workdir, seedval = seed.val, 
-                          overwrite = TRUE)
-      if (do.stack) {
-        if (proj_is_raster) {
-          return(wrap(pred.tmp)) 
-        } else {
-          return(pred.tmp)
-        }
-      } else {
-        return(filename)
-      }
+    cat("\n\t> Projecting", mod.name, "...")
+    if (do.stack) {
+      filename <- NULL
+    } else {
+      filename <- file.path(namePath, "individual_projections",
+                            paste0(nameProj, "_", mod.name, 
+                                   ifelse(output.format == ".RData"
+                                          , ".tif", output.format)))
     }
-  
+    
+    mod <- get(BIOMOD_LoadModels(bm.out = bm.mod, full.name = mod.name))
+    temp_workdir = NULL
+    if (length(grep("MAXENT$", mod.name)) == 1) {
+      temp_workdir = mod@model_output_dir
+    }
+    pred.tmp <- predict(mod, new.env, on_0_1000 = on_0_1000, 
+                        filename = filename, omit.na = omit.na, 
+                        temp_workdir = temp_workdir, seedval = seed.val, 
+                        overwrite = TRUE, mod.name = mod.name)
+    if (do.stack) {
+      if (proj_is_raster) {
+        return(wrap(pred.tmp)) 
+      } else {
+        return(pred.tmp)
+      }
+    } else {
+      return(filename)
+    }
+  }
   ## Putting predictions into the right format
-  if (do.stack){
+  if (do.stack) {
     if (proj_is_raster) {
-      proj <- rast(lapply(proj, rast)) # SpatRaster needs to be wrapped before saving
+      proj <- rast(lapply(proj, unwrap)) # SpatRaster needs to be wrapped before saving
       names(proj) <- models.chosen
       proj <- wrap(proj)
       proj.trans <- proj
@@ -342,7 +342,7 @@ BIOMOD_Projection <- function(bm.mod,
   }
   
   ## save projections
-  proj_out@type <- class(new.env)
+  proj_out@type <- .get_env_class(new.env)
   if (!do.stack){
     saved.files = unlist(proj)
   } else {
@@ -503,16 +503,22 @@ BIOMOD_Projection <- function(bm.mod,
   ## 3. Check new.env ---------------------------------------------------------
   .fun_testIfInherits(TRUE, "new.env", new.env, c('matrix', 'data.frame', 'SpatRaster','Raster'))
   
-  if(inherits(new.env, 'matrix')){
+  if (inherits(new.env, 'matrix')) {
     if (any(sapply(get_formal_data(bm.mod, "expl.var"), is.factor))) {
       stop("new.env cannot be given as matrix when model involves categorical variables")
     }
     new.env <- data.frame(new.env)
+  } else if (inherits(new.env, 'data.frame')) {
+    # ensure that data.table are coerced into classic data.frame
+    new.env <- as.data.frame(new.env) 
   }
+  
   if (inherits(new.env, 'Raster')) {
     # conversion into SpatRaster
-    if(any(raster::is.factor(new.env))){
-      new.env <- categorical_stack_to_terra(raster::stack(new.env))
+    if (any(raster::is.factor(new.env))) {
+      new.env <- .categorical_stack_to_terra(raster::stack(new.env),
+                                            expected_levels = head(get_formal_data(bm.mod, subinfo = "expl.var"))
+      )
     } else {
       new.env <- rast(new.env)
     }
@@ -524,6 +530,12 @@ BIOMOD_Projection <- function(bm.mod,
     .fun_testIfIn(TRUE, "colnames(new.env)", colnames(new.env), bm.mod@expl.var.names)
   }
   
+  which.factor <- which(sapply(new.env, is.factor))
+  if (length(which.factor) > 0) {
+    new.env <- .check_env_levels(new.env, 
+                                 expected_levels = head(get_formal_data(bm.mod, subinfo = "expl.var")))
+  }
+  
   ## 4. Check new.env.xy ------------------------------------------------------
   if (!is.null(new.env.xy)  & !inherits(new.env, 'SpatRaster')) {
     new.env.xy = as.data.frame(new.env.xy)
