@@ -1,7 +1,6 @@
 
 ##' @name BIOMOD.stored.data
 ##' @aliases BIOMOD.stored.data-class
-##' @aliases BIOMOD.stored.array-class
 ##' @aliases BIOMOD.stored.data.frame-class
 ##' @aliases BIOMOD.stored.SpatRaster-class
 ##' @aliases BIOMOD.stored.files-class
@@ -29,7 +28,6 @@
 ##' contain a \code{val} slot of specific type :
 ##' 
 ##' \itemize{
-##'   \item{\code{BIOMOD.stored.array} : }{\code{val} is an \code{array}}
 ##'   \item{\code{BIOMOD.stored.data.frame} : }{\code{val} is a \code{data.frame}}
 ##'   \item{\code{BIOMOD.stored.SpatRaster} : }{\code{val} is a 
 ##'   \code{\link[terra:PackedSpatRaster-class]{PackedSpatRaster}}}
@@ -53,7 +51,6 @@
 ##' @examples 
 ##' 
 ##' showClass("BIOMOD.stored.data")
-##' showClass("BIOMOD.stored.array") 
 ##' showClass("BIOMOD.stored.data.frame") 
 ##' showClass("BIOMOD.stored.SpatRaster") 
 ##' showClass("BIOMOD.stored.files") 
@@ -81,16 +78,6 @@ setClass("BIOMOD.stored.data",
          prototype(inMemory = FALSE, link = ''),
          validity = function(object){ return(TRUE) })
 
-### BIOMOD.stored.array ------------------------------------------------
-##' @name BIOMOD.stored.array-class
-##' @rdname BIOMOD.stored.data
-##' 
-setClass("BIOMOD.stored.array",
-         contains = "BIOMOD.stored.data",
-         representation(val = 'array'),
-         prototype(val = array()),
-         validity = function(object){ return(TRUE) })
-
 ### BIOMOD.stored.data.frame ------------------------------------------------
 ##' @name BIOMOD.stored.data.frame-class
 ##' @rdname BIOMOD.stored.data
@@ -221,22 +208,6 @@ setMethod("load_stored_object", "BIOMOD.stored.SpatRaster",
                            grepl(".img", obj@link) | 
                            grepl(".tif", obj@link))) {
               out <- rast(x = current_link)
-              ## rename layer in case of individual projections
-              if (all(grepl("individual_projections", current_link))) {
-                # remove directories arch and extension
-                if (any(grepl("bin", current_link)) | 
-                    any(grepl("filt", current_link)) ) {
-                  xx <- sub("_[^_]+$", "", current_link)
-                  xx <- sub("^.+individual_projections/", "", xx)
-                } else {
-                  xx <- sub("[:.:].+$", "",
-                            sub("^.+individual_projections/", "", current_link))
-                }
-                # remove projection name
-                to_rm <- unique(sub("[^_]+[:_:][^_]+[:_:][^_]+[:_:][^_]+$", "", xx))
-                xx <- sub(to_rm, "", xx)
-                names(out) <- xx
-              }
               return(out)
             }
           }
